'use strict';
const util = require("util");
var DBusServer = require("./DBusServer.js");
const CONSTS = require("./ProxyConsts.js");
const config = require('./config');
const Constant = require("../../../utils/constant");
const Log = require("../../../utils/log");
const TAG = "dbus_EdgeSideServer";
var broker = require("../../broker/index");

var EdgeSideServer = function () {
    // this.server = new DBusServer("com.ali.iot.edge.liangqin", "/com/ali/iot/edge/liangqin", "com.ali.iot.edge.liangqin");
    this.server = new DBusServer("iot.runtime.js", "/iot/runtime/js", "iot.runtime.js");

    if (!this.server.prepare()) {
//        console.log("prepare EdgesideServer failed. quit.");
        process.exit(1);
    }
    // JobManager signals
    this.server.addMatch("type='signal',path='/iot/job/manager',interface='iot.job.manager'");
    // DMP signals
    this.server.addMatch("type='signal',path='/iot/dmp/dimu',interface='iot.dmp.dimu'");
    if (config.RUN_MODE === config.MODE_OPTION.MOCK_PROXY_REAL_SERVER_WITH_ALL_DMSG) {
        // for test receive all signal
        this.server.addMatch("type='signal'");
   }
    this.server.addMatch("type='method_call',path='/iot/runtime/js',interface='iot.runtime.js'");

    /**
     * 接收发送到EdgeSide的signal，每个发送过来的消息参数args格式如下：
     * ［id, value]
     * @param  {[type]} signal [description]
     * @param  {[type]} args)  {               } [description]
     * @return {[type]}        [description]
     */
    var self = this;
    // this.server.on("signal", receiveSignalWrap(self));
    this.server.on("signal", receiveSignalWrap);

    this.server.on("method_call", receiveMethodCall);
    this.server.on("method_call_with_reply", receiverMethodCallWithReply);
    this.server.start();
};

// function receiveSignalWrap(self) {
  function receiveSignalWrap(signal, sender, args) {
      Log.i(`EdgeSideServer receive signal: ${signal} sender: ${sender} arg: ${args}`);
    // console.log(`EdgeSideServer receive signal: ${signal} sender: ${sender} arg: ${args}`);
    if (sender === "/org/freedesktop/DBus") return;

    /*
     * CMP发送的propertiesChanged和设备Events发生，预处理为以下形式
     * (按照约定，CMP只发properties changed和设备的events，不发任何其它signal)：
     * 1.　属性变化：　propertiesChanged
     *     原始signal格式：
     *        　{
     *        　    key: value
     *        　}
     *
     *     参数格式：　
     *             {
     *                 "id": "cloudid",
     *                 "params": {
     *                         "key1": "value1",
     *                         "key2": "value2",
     *                         ...
     *                     }
     *             }
     *
     * 2. Events事件：  ThingEventHappened
     *    原始格式：
     *    sinal即事件名：
     *    ｛
     *        params: {
     *            key: value
     *        }
     *    参数格式：
     *            {
     *                "id": "cloudid",
     *                "eventName": "real_event_name",
     *                "params": {
     *                        // event的参数，和profile中描述一致
     *                    }
     *            }
     *
     */
    var msg = null;
    try {
      if(signal.startsWith("FC")) {
          msg = {};
          //msg[Constant.MESSAGE_NAME] = signal.split("_")[1];
          msg.event = {};
          msg[Constant.CONFIG_FUNCTIONID] = signal.split("_")[0].substr('FC'.length);
          (msg.event)["payload"] = args;
          (msg.event)["topic"] = sender;
          signal = "Filter_Event";
      } else if (sender.startsWith(CONSTS.CmpPrefix)) {
        var payload = JSON.parse(args);
        if (Object.prototype.toString.call(payload) !== '[object Object]') {
            Log.e(TAG, 'CMP message received: illegal arguments');
          return;
        }
        var mystr = sender.substr(CONSTS.CmpPrefix.length);
        var pk = mystr.split("/")[0];
        var dn = (mystr.split("/")[1]).split("/")[0];
        var id = pk + "_" + dn;
        if (CONSTS.PropertiesChanged === signal) {
          var params = payload;
          msg = {'id' : id, 'params' : params};
        } else {
          var params = payload.params;
          msg = {'id' : id, 'params' : params};
          msg.eventName = signal;
          signal = CONSTS.ThingEventHappened;
        }
      } else if ('deviceStateChanged' === signal) {
        var payload = JSON.parse(args);
        if (payload.code !== 0) {
          Log.e(TAG, 'Thing state changed: error, ' + payload.message);
          return;
        }
        if (payload.params === null || payload.params.state === null ||
          payload.params.devList === null) {
            Log.e(TAG, 'Thing state changed: illegal arguments');
          return;
        }
        if (payload.params.state === 'online') {
          signal = CONSTS.ThingConnected;
        } else if (payload.params.state === 'offline') {
          signal = CONSTS.ThingDisconnected;
        } else {
          Log.e(TAG, 'Thing state changed: unsupported state');
          return;
        }
        msg = payload.params.devList;
      } else {
        msg = JSON.parse(args);
      }
    } catch (e) {
        Log.e(util.format("EdgeSideServer parse signla's args: '%s' failed: %s", args, e.message));
        return;
    }
    // Log.i(TAG, 'start do dispatch signal:', signal, 'msg:', JSON.stringify(msg));
    // if (self.signalHandler) {
    //   self.signalHandler(signal, msg);
    // }
    broker.publish(signal, msg);
    msg = null;
  }

function receiveMethodCall(method, args) {
  Log.i(TAG, "EdgeSideServer on method call: " + method + " args: " + args);


}

function receiverMethodCallWithReply(method, args, reply) {
    Log.i(TAG, "receiverMethodCallWithReply on method : " + method + " args: " + args + "reply :" + JSON.stringify(reply));
    if(method == "resetJobs") {
        broker.publish("resetJobs");
        var res = {};
        res.code = 0;
        res.message = "reset ifttt success";
        res.data = {};
        return JSON.stringify(res);
    }
}


EdgeSideServer.prototype.registerSignalHandler = function (handler) {
    this.signalHandler = handler;
};

EdgeSideServer.prototype.sendSignal = function (signal, args) {
    return this.server.sendSignal(signal, args);
};

EdgeSideServer.prototype.traceLog = function (level, tag, file, func, line, log) {
    this.server.traceLog(level, tag, file, func, line, log);
};

module.exports = new EdgeSideServer();
