/*
{
    "name" : "hotel",
	"groupId" : 1345,

    "devices" : [
        {"pk": "XX", "dn": "aa0", "tags":[{"key":"room", "value":"101"}, {"key":"deviceType", "value":"lock"}]},
        {"pk": "YY", "dn": "bb0", "tags":[{"key":"room", "value":"101"}, {"key":"deviceType", "value":"humanDector"}]},
        {"pk": "ZZ", "dn": "cc0", "tags":[{"key":"room", "value":"101"}, {"key":"deviceType", "value":"light"}, {"key":"location", "value":"livingRoom"}]},
        {"pk": "ZZ", "dn": "dd0", "tags":[{"key":"room", "value":"101"}, {"key":"deviceType", "value":"light"}, {"key":"location", "value":"balcony"}]}
    ]
}
 */
var register = require("../registry/registerDevice");
const dataCenter = require("../registry/DataCenter");
var groupAPI = require("./api-group");
var Log = require("./log");
const TAG = "developer_Group";

/**
 * 根据groupId 生成分组对象
 * @param {String} groupId
 * @return {undefined}
 * @constructor
 */
var Group = function (groupId) {
    Log.LOGI(TAG, "groupId = " + groupId);
    if(!groupId || groupId < 0) {
        return undefined;
    }
    this.groupId = groupId;
    this.getConfigSucceed = false;
    this.initDevieSucceed = false;
};

/**
 * 根据productKey, deviceName 生成设备对象
 * @param {String} productKey
 * @param {String} deviceName
 * @param {Function} callback(Error err, DynamicDevice device)
 */
Group.prototype.getDeviceByProductKeyDeviceName = function (productKey, deviceName, callback) {
    Log.LOGI(TAG, "getDeviceByProductKeyDeviceName begin productKey :", productKey, "deviceName :", deviceName);
    if(arguments.length != 3 || typeof arguments[2] != 'function') {
        Log.LOGE(TAG, "arguments[2] must be callback");
        throw new Error("arguments[2] must be callback");
        return;
    }
    if(!productKey || !deviceName || productKey === '' || deviceName === '') {
        return callback(new Error('params error'));
    }
    var self = this;
    this.isValidDevice(productKey, deviceName, function (err, status) {
        Log.LOGI(TAG, productKey, deviceName, " isValidDevice err =", err, "status = ", status);
        if(!err && status) {
            self.getTagsByDevice(productKey, deviceName, function (err1, tags) {
                register.createThing(productKey + "_" + deviceName, tags, function (err2, thing) {
                    return callback(err2, thing);
                });
            });
        } else {
            if(!err && !status) {
                err = new Error('invalidDevice');
            }
            Log.LOGI(TAG, "isValidDevice err = ",err);
            return callback(err, null);
        }
    });
};

Group.prototype.isValidDevice = function (pk, dn, callback) {
    var self = this;
    if(arguments.length != 3 || typeof arguments[2] != 'function') {
        Log.LOGE(TAG, "arguments[2] must be callback");
        throw new Error("arguments[2] must be callback");
        return;
    }
    if(!pk || !dn || pk === '' || dn === '') {
        return callback(new Error('params error'));
    }
    if(this.getConfigSucceed === false) {
        this.getConfig(function (err) {
            Log.LOGI(TAG, "isValidDevice getConfig err", err);
            if(!err) {
                return callback(null, self.isValidDeviceAfterInit(pk, dn));
            }
            return callback(err);
        });
    } else {
        return callback(null, this.isValidDeviceAfterInit(pk, dn));
    }
};

Group.prototype.isValidDeviceAfterInit = function(pk, dn) {
    Log.LOGI(TAG, "isValidDeviceAfterInit pk :", pk, "dn :", dn);
    Log.LOGI(TAG, "isValidDeviceAfterInit deviceJsonObjects ", JSON.stringify(this.deviceJsonObjects));
    if(!pk || !dn) return false;
    if(!this.deviceJsonObjects) return false;
    for(var deviceObject of this.deviceJsonObjects) {
        if(deviceObject.productKey === pk && deviceObject.deviceName === dn) {
            return true;
        }
    }
    return false;
};

Group.prototype.getDevicesByAliasName = function (aliasName) {
    var devices = dataCenter.getDeviceObjectList();
    if( !device) {
        return null;
    }
    for(var device of devices) {
        if(device.getName() === aliasName) {
            return device;
        }
    }
};

/**
 * 判断aTags 是否包含bTags
 * @param {object} aTags
 * @param {object} bTags
 * @return {boolean}
 * eg:[{}, {}, {}] [{}, {}]
 */
function isContains(aTags, bTags) {
    if(aTags == null && bTags == null ) {
        return true;
    } else if(!aTags || !bTags) {
        return false;
    }
    if(Array.isArray(aTags) && Array.isArray(bTags)) {
        var cntBTag = 0;
        for(var bTag of bTags) {
            for(var aTag of aTags) {
                var cnt = 0;
                for(var key in bTag) {
                    if(aTag.hasOwnProperty(key) && aTag[key] === bTag[key]) {
                        cnt++;
                    }
                }
                if(cnt === Object.keys(bTag).length) {
                    cntBTag++;
                    break;
                }
            }
        }
        if(cntBTag === Object.keys(bTags).length) {
            return true;
        }
    }
    return false;
}

/**
 * 设备组设备初始化后get 满足某一个tag要求的设备列表
 * @param  {Array} tags eg: [{}, {}]
 * @param  {Function} callback(err, [])
 * @return {Array}
 */
Group.prototype.getDevicesByTagAfterInit = function (tags, callback) {
    var devices = [];
    var self = this;
    if(!tags || !Array.isArray(tags) ||tags.length === 0) return callback(new Error('params Error, tags type is Array, your tags = ' + tags));
    var waitStatus = false;
    for(var i = 0; i < this.deviceJsonObjects.length; i++) {
        var deviceObject = (this.deviceJsonObjects)[i];
        var tagsArray = deviceObject.tags ? deviceObject.tags : [];
        if(isContains(tagsArray, tags)) {
            waitStatus = true;
            register.createThing(deviceObject.productKey + "_" + deviceObject.deviceName, tagsArray, function (err, thing) {
                waitStatus = false;
                if(!err) {
                    devices.push(thing);
                } else {
                    Log.LOGE(TAG, "getDevicesByTagAfterInit createThing err =" + err);
                }
                if(i >= self.deviceJsonObjects.length - 1) {
                    return callback(null, devices);
                }
            });
        }
    }
    if(!waitStatus) {
        return callback(null, devices);
    }
};

/**
 * 获取符合一组Tag要求的设备列表
 * @param {Array} tags
 * @param {Function} callback(Error err, List<DynamicDevice> list)
 */
Group.prototype.getDevicesByTag = function(tags, callback) {
    if(arguments.length != 2 || typeof arguments[1] != 'function') {
        Log.LOGE(TAG, "arguments[1] must be callback");
        throw new Error("arguments[1] must be callback");
        return;
    }
    if(!tags || !Array.isArray(tags)) {
        return callback(new Error('params error'));
    }
    var devices = [];
    var self = this;
    if(this.getConfigSucceed === false) {
        this.getConfig(function (err, list) {
            Log.LOGI(TAG, "getDevicesByTag getConfig err", err, 'list',list);
            if(!err) {
                self.getDevicesByTagAfterInit(tags, function (err1, list) {
                    return callback(err1, list);
                });
            } else {
                return callback(err);
            }
        });
    } else {
        this.getDevicesByTagAfterInit(tags, function (err2, list) {
            return callback(err2, list);
        });
    }
};

/**
 * 获取某一类设备列表
 * @param {String} productKey
 * @param {Function} callback(Error err, List<DynamicDevice> devices)
 * @return {*}
 */
Group.prototype.getDevicesByProductKey = function (productKey, callback) {
    Log.LOGI(TAG, "getDevicesByProductKey begin productKey = ", productKey);
    if(arguments.length != 2 || typeof arguments[1] != 'function') {
        Log.LOGE(TAG, "arguments[1] must be callback");
        throw new Error("arguments[1] must be callback");
        return;
    }
    if(!productKey || productKey === "") {
        return callback(new Error('params error'));
    }
    var devices = [];
    var self = this;
    if(this.initDevieSucceed === false) {
        this.initDevices(function (err) {
            Log.LOGI(TAG, "getDevicesByProductKey  initDevices  err = ", err);
            if(!err) {
                Log.LOGI(TAG, "getDevicesByProductKey  initDevices  deviceRoot = ", JSON.stringify(self.devicesRoot));
                if(self.devicesRoot && self.devicesRoot.devices) {
                    return callback(null, (self.devicesRoot.devices)[productKey]);
                }
                return callback(null, devices);
            } else {
                return callback(err);
            }
        });
    } else {
        Log.LOGI(TAG, "getDevicesByProductKey deviceRoot = ", JSON.stringify(self.deviceRoot));
        if(this.devicesRoot && this.devicesRoot.devices) {
            return callback(null, (this.devicesRoot.devices)[productKey]);
        } else {
            return callback(null, devices);
        }
    }
};

/**
 * 获取分组列表下符合 tagKey 的 Tag value列表
 * @param {string} tagKey
 * @param {Function} callback(Error err, List<String> tagValues)
 */
Group.prototype.getTagValueList = function (tagName, callback) {
    Log.LOGI(TAG, "getTagValueList tagName =", tagName);
    if(arguments.length != 2 || typeof arguments[1] != 'function') {
        Log.LOGE(TAG, "arguments[1] must be callback");
        throw new Error("arguments[1] must be callback");
        return;
    }
    if(!tagName || tagName === '') {
        return callback(new Error('params error'));
    }
    var tagList = [];
    var self = this;
    if(this.getConfigSucceed === false) {
        this.getConfig(function (err) {
            Log.LOGI(TAG, "getTagValueList getConfig err", err);
            if(!err) {
                return callback(err, self.getTagValueListAfterInit(tagName));
            } else {
                return callback(err);
            }
        })
    } else {
        return callback(null, this.getTagValueListAfterInit(tagName));
    }
};


Group.prototype.getTagValueListAfterInit = function (tagName) {
    var tagList = [];
    if(this.deviceJsonObjects) {
        for(var json of this.deviceJsonObjects) {
            var tagArray = json.tags ? json.tags : [];
            if(JSON.stringify(tagArray).indexOf(tagName) != -1) {
                for(var i = 0 ; i < tagArray.length; i++) {
                    var tag = tagArray[i];
                    if(tag.key === tagName) {
                        tagList.push(tag.value);
                    }
                }
            }
        }
    }
    return tagList;
};

/**
 * 获取分组列表下符合 tagKey 的 Tag 列表
 * @param {String} tagName
 * @param {Function} callback(Error err, List<Tag> tags)
 */
Group.prototype.getTagList = function (tagName, callback) {
    Log.LOGI(TAG, "getTagList tagName =", tagName);
    if(arguments.length != 2 || typeof arguments[1] != 'function') {
        Log.LOGE(TAG, "arguments[1] must be callback");
        throw new Error("arguments[1] must be callback");
        return;
    }
    if(!tagName || tagName == '') {
        return callback(new Error('params error'));
    }
    var tagList = [];
    var self = this;
    if(this.getConfigSucceed === false) {
        this.getConfig(function (err) {
            Log.LOGI(TAG, "getTagList getConfig err", err);
            if(!err) {
                return callback(err, self.getTagAfterInit(tagName));
            } else {
                return callback(err);
            }
        });
    } else {
        return callback(null, this.getTagAfterInit(tagName));
    }
};

Group.prototype.getTagAfterInit = function (tagName) {
    var tagList = [];
    if(this.deviceJsonObjects) {
        for(var json of this.deviceJsonObjects) {
            var tagArray = json.tags ? json.tags : [];
            if(JSON.stringify(tagArray).indexOf(tagName) != -1) {
                for(var i = 0 ; i < tagArray.length; i++) {
                    var tag = tagArray[i];
                    if(tag.key === tagName) {
                        tagList.push(tag);
                    }
                }
            }
        }
    }
    return tagList;
};

/**
 * 获取分组配置中的设备tags
 * @param pk
 * @param dn
 * @param callback
 * @return {*}
 */
Group.prototype.getTagsByDevice = function (pk, dn, callback) {
    Log.LOGI(TAG, "getTagsByDevice productKey", pk, "deviceName", dn);
    if(arguments.length != 3 || typeof arguments[2] != 'function') {
        Log.LOGE(TAG, "arguments[2] must be callback");
        throw new Error("arguments[2] must be callback");
        return;
    }
    if(!pk || !dn || pk === '' || dn === '') {
        return callback(new Error('params error'));
    }
    var tags = [];
    var self = this;
    if(this.getConfigSucceed === false) {
        this.getConfig(function (err) {
            Log.LOGI(TAG, "getTagsByDevice getConfig err", err);
            if(!err) {
                return callback(err, self.getTagsByDeviceAfterInit(pk, dn));
            } else {
                return callback(err);
            }
        })
    } else {
        return callback(null, this.getTagsByDeviceAfterInit(pk, dn));
    }
};

Group.prototype.getTagsByDeviceAfterInit = function (productKey, deviceName) {
    for(var i = 0; i < this.deviceJsonObjects.length; i++) {
        var deviceObject = this.deviceJsonObjects[i];
        if(deviceObject.productKey === productKey && deviceObject.deviceName === deviceName) {
            return deviceObject.tags;
        }
    }
    return [];
};

/**
 * 强制初始化所有设备
 * @param callback
 */
Group.prototype.initDevices = function (callback) {
    Log.LOGI(TAG, "initDevices begin...");
    var self = this;
    groupAPI.getThingsByGroupId(this.groupId, function (err, thing) {
        Log.LOGI(TAG, "initDevices getThingsByGroupId err =", err, "thing = ", JSON.stringify(thing));
        if(!err) {
            self.devicesRoot = thing;
            self.config = groupAPI.config;
            self.deviceJsonObjects = (self.config && self.config.devices) ? self.config.devices : [];
            self.getConfigSucceed = true;
            self.initDevieSucceed = true;
        }
        if(!self.deviceJsonObjects) {
            self.deviceJsonObjects = [];
        }
        callback(err, thing);
    });
};

/**
 * 强制获取全量groupConfig
 * @param callback
 */
Group.prototype.getConfig = function (callback) {
    var self = this;
    groupAPI.getConfigByGroupId(this.groupId, function (err, config) {
        Log.LOGI(TAG, "getConfig getConfigByGroupId err =", err, "config = ", JSON.stringify(config));
        if(!err) {
            self.config = config;
            self.deviceJsonObjects = (self.config && self.config.devices) ? self.config.devices : [];
            self.getConfigSucceed = true;
        }
        if(!self.deviceJsonObjects) {
            self.deviceJsonObjects = [];
        }
        return callback(err, config);
    });
};

module.exports = Group;
