/**
 * "action":[
        {
            "node":"action/device/setStatus",
            "params":{
                "productKey":"",
                "deviceName":"",
                "propertyItems":{
                    "attr":"value",
                    "attr2":"value2"
                }
            }
        }
    ]
 */
const Tag = require('../../utils/constant');
const log = require('../developer/log');
const fs = require('fs');
const path = require('path');
const LOGTAG = "IFTTT::Action";
const util = require('util');
var condition = require("./condition");
const { PROJECT_PATH } = require('../../const');

var Action = function(actions, sid, proxy, traceId) {
    this.sceneId = sid;
    this.proxy = proxy;
    this.actions = actions;
    this.traceId = traceId;
};

/** DETAIL 日志 ['uri', 'bizCode', 'traceId', 'sceneId', 'code', 'msg', 'timestamp', ...,
 * 'productKey', 'deviceName', 'propertyItems', 'serviceName', 'serviceArgs']
 * [2018-05-02 17:46:00]; uri:action/device/setProperty; bizCode:DETAIL; traceId:0a6550ce15252543600081930d1767; sceneId:a111e96fe83b44348e8a7ed9864a0c48; code:200; msg:success; timestamp:1525254360452; productKey:a1NfftnCLGg; deviceName:wudSNSEccE0ppPhKi88X; propertyItems:{"int_rw":5}
 */
Action.prototype.generateLog = function (code, msg, uri, productKey, deviceName, propertyItems, serviceName, serviceArgs) {
    var log = util.format("[%s]", (new Date).toLocaleString());
    log += ";" + "uri" + ":" + uri;
    log += ";" + "bizCode" + ":" + "DETAIL";
    log += ";" + "traceId" + ":" + this.traceId;
    log += ";" + "sceneId" + ":" + this.sceneId;
    log += ";" + "code" + ":" + code;
    log += ";" + "msg" + ":" + msg.toString();
    log += ";" + "timestamp" + ":" + Math.round(new Date().getTime());// 毫秒
    if(productKey) {
        log += ";" + "productKey" + ":" + productKey;
    }
    if(deviceName) {
        log += ";" + "deviceName" + ":" + deviceName;
    }
    if(propertyItems) {
        log += ";" + "propertyItems" + ":" + JSON.stringify(propertyItems);
    }
    if(serviceName) {
        log += ";" + "serviceName" + ":" + serviceName;
    }
    if(serviceArgs) {
        log += ";" + "serviceArgs" + ":" + JSON.stringify(serviceArgs);
    }
    log += "\n";
    return log;
};

Action.prototype.execute = function (callback) {
    log.LOGI(LOGTAG, "execute begin sid = " + this.sceneId);
    var self = this;
    var actionsLog = [];
    if(!this.actions || this.actions.length === 0) return callback(new Error('actions empty'));
    var hasReturned = false;
    var cnt = 0;
    var returnErr = false;
    log.LOGI(LOGTAG, "execute actions size = ", this.actions.length);
        this.actions.forEach(function(action) {
        if (!action || !(action)[Tag.PARAMS] || ! action[Tag.URI]) {
            var msg = "action sid " + self.sceneId + " params error, action = " + JSON.stringify(action);
            log.LOGE(LOGTAG, msg);
            actionsLog.push(self.generateLog(Tag.RUN_FAIL, msg, null, null, null, null, null, null));
            cnt++;
            returnErr = true;
            return;
        }
        var params = action[Tag.PARAMS];
        var actionUri = action[Tag.URI];
        log.LOGI(LOGTAG, "execute actionUri = " + actionUri);
        if (actionUri === Tag.ActionSetDevice || actionUri === Tag.ActionSetProperty) {
            var pk = params[Tag.PRODUCTKEY];
            var dn = params[Tag.DEVICENAME];
            var id = pk + "_" + dn;
            var attr = params[Tag.PROPERTYITEMS];
            if (attr) {
                log.LOGI(LOGTAG, "setProperty", id, JSON.stringify(attr));
                self.proxy.setProperty(id, attr, function(err, value) {
                    log.LOGI(LOGTAG, 'execute sid =', self.sceneId, actionUri, 'attr', JSON.stringify(attr), 'result: err =', err, 'value =', JSON.stringify(value));
                    var code = Tag.RUN_SUCCESS;
                    var msg = "success";
                    if(err) {
                        returnErr = err;
                        code = Tag.RUN_FAIL;
                        msg = err.toString();
                    }
                    actionsLog.push(self.generateLog(code, msg, action[Tag.URI], pk, dn, attr, null, null));
                    cnt++;
                    if(!hasReturned && (cnt === self.actions.length)) {
                        hasReturned = true;
                        return callback(returnErr, actionsLog);
                    }
                });
            } else {
                cnt++;
                var mess = 'propertyItems for action is not defined.';
                log.LOGE(LOGTAG, mess);
                returnErr = true;
                actionsLog.push(self.generateLog(Tag.RUN_FAIL, mess, action[Tag.URI]), pk, dn, attr, null, null);
            }
        } else if (action[Tag.URI] === Tag.ActionInvokeService) {
            var pk = params[Tag.PRODUCTKEY];
            var dn = params[Tag.DEVICENAME];
            var id = pk + "_" + dn;
            var srvName = params[Tag.ServiceName];
            var srvArgs = params[Tag.ServiceArgs];
            if (srvName && srvArgs) {
                log.LOGI(LOGTAG, "callService", id, srvName, JSON.stringify(srvArgs));
                self.proxy.callService(id, srvName, srvArgs, function(err, value) {
                    log.LOGI(LOGTAG, 'execute sid = ', self.sceneId, Tag.ActionInvokeService, 'result: err = ', err, 'value =', JSON.stringify(value));
                    var code = Tag.RUN_SUCCESS;
                    var msg = "success";
                    if(err) {
                        returnErr = err;
                        code = Tag.RUN_FAIL;
                        msg = err.toString();
                    }
                    actionsLog.push(self.generateLog(code, msg, action[Tag.URI], pk, dn, null, srvName, srvArgs));
                    cnt++;
                    if(!hasReturned && (cnt === self.actions.length)) {
                        hasReturned = true;
                        return callback(returnErr, actionsLog);
                    }
                });
            } else {
                cnt++;
                var mess = 'action execute　sid = ' + self.sceneId + action[Tag.URI] + ' failed for: ' + Tag.ServiceName + '=' + srvName +  Tag.ServiceArgs + " = " + srvArgs;
                log.LOGE(LOGTAG, mess);
                returnErr = true;
                actionsLog.push(self.generateLog(Tag.RUN_FAIL, mess, action[Tag.URI], pk, dn, null, srvName, srvArgs));
            }
        } else if (action[Tag.URI] === Tag.ActionScence) {
            var sid = params[Tag.SCENEID];
            log.LOGI(LOGTAG, Tag.ActionScence, 'findTargetSence begin sid =', sid);
            findTargetSence(sid, function (err, sceneObject, jobInfo) {
                var mess = 'findTargetSence ' +  sid +  ' err =' + err + " sceneObject = " + JSON.stringify(sceneObject);
                log.LOGI(LOGTAG, mess);
                if (err || !sceneObject) {
                    cnt++;
                    log.LOGE(LOGTAG, mess);
                    returnErr = err;
                    actionsLog.push(self.generateLog(Tag.RUN_FAIL, 'sid ' + sid + " can't find, err = " + err.toString(), action[Tag.URI], null, null, null, null, null));
                    if(!hasReturned && (cnt === self.actions.length)) {
                        hasReturned = true;
                        return callback(returnErr, actionsLog);
                    }
                } else {
                    var conditionObject =[];
                    if (sceneObject.hasOwnProperty(Tag.CONDITION)) {
                        var conditionObject = sceneObject[Tag.CONDITION];
                        if (conditionObject['items'] != undefined) {
                            conditionObject = conditionObject['items'];
                        } else {
                            var items = [];
                            items.push(conditionObject);
                            conditionObject = items;
                        }
                    }
                    log.LOGI(LOGTAG, "isConditionsMeeted condition = " + JSON.stringify(conditionObject));
                    condition.isConditionsMeeted(conditionObject, function (err, satisfy) {
                        log.LOGI(LOGTAG, "isConditionsMeeted err = " + err + " satisfy =" + satisfy);
                        if (!err && satisfy) {
                            log.LOGI(LOGTAG, "Subscene condition satisfy begin to execute");
                            var actionInner = new Action(sceneObject[Tag.ACTION], self.sceneId, self.proxy, self.traceId);
                            actionInner.execute(function (err, status) {
                                log.LOGI(LOGTAG, "actionInner execute callback err = " + err + " status =" + status);
                                cnt++;
                                var code = Tag.RUN_SUCCESS;
                                var mess = "success";
                                if(err) {
                                    returnErr = err;
                                    code = Tag.RUN_FAIL;
                                    mess = err.toString();
                                    log.LOGE(LOGTAG, mess);
                                }
                                //actionsLog.push(self.generateLog(code, mess, action[Tag.URI], null, null, null, null, null)); quit subscene report itself status
                                actionsLog.push(status);
                                if(!hasReturned && (cnt === self.actions.length)) {
                                    hasReturned = true;
                                    return callback(returnErr, actionsLog);
                                }
                            });
                        } else if(err) {
                            cnt++;
                            var mess = 'isConditionsMeeted err = ' + err.toString();
                            log.LOGE(LOGTAG, mess);
                            returnErr = err;
                            actionsLog.push(self.generateLog(Tag.RUN_FAIL, mess, action[Tag.URI]));
                            if(!hasReturned && (cnt === self.actions.length)) {
                                hasReturned = true;
                                return callback(returnErr, actionsLog);
                            }
                        } else if(!satisfy) {
                            log.LOGI(LOGTAG, "sid", sid, "isConditionsMeeted  satisfy =" + satisfy, "will not execute");
                            cnt++;
                            if(!hasReturned && (cnt === self.actions.length)) {
                                hasReturned = true;
                                return callback(returnErr, actionsLog);
                            }
                        }
                    });
                }
            });
        } else {
            cnt++;
            var mess = action[Tag.URI] + ' is not support yet.';
            log.LOGE(LOGTAG, mess);
            returnErr = true;
            actionsLog.push(self.generateLog(Tag.RUN_FAIL, mess, action[Tag.URI]));
        }
    });
    if(cnt === this.actions.length && !hasReturned) {
        hasReturned = true;
        return callback(returnErr, actionsLog);
    }
};

/**
 * @return {JsonObject} 找到目标sid 对应的场景内容
 * @param sid
 * @param callback(err, jsonContent)
 */
function findTargetSence(sid, callback) {
    if(!sid) {
        return callback(new Error('params sid Error'));
    }
    var configPath = path.join(Tag.FOTA_IFTTT, "configuration.json");
    fs.readFile(configPath, 'utf8', function (err, data) {
        var hasReturn = false;
        if(!err) {
            try {
                var configFileObject = JSON.parse(data);
                var cnt = 0;
                configFileObject.forEach(function(config) {
                    var jobPath = path.join(config.path, Tag.jobIndex);
                    fs.readFile(jobPath, 'utf8', function (err, data) {
                        cnt++;
                        try {
                            var content = JSON.parse(data);
                            if(!hasReturn && content.sid === sid) {
                                hasReturn = true;
                                return callback(null, content, config);
                            }
                        } catch (e) {
                            log.LOGE(LOGTAG, 'findTargetSence', jobPath, 'parse err =', e, 'data =', data);
                        }
                        if(!hasReturn && cnt === configFileObject.length) {
                            hasReturn = true;
                            return callback(true);
                        }
                    });
                });
            } catch (e) {
                hasReturn = true;
                log.LOGE(LOGTAG, 'findTargetSence', configPath, 'parse err =', e);
                return callback(e);
            }
        } else {
            hasReturn = true;
            return callback(err);
        }
    });
}

module.exports = Action;
