'use strict';
/*var config = [{
  "jobId": 0,
  "ackCode" : 10201,
  "path": "/workspace/xiaomin.wxm/Projects/iot-gravity/loader/test",
  "type" : "xxx",
  "executor" : "NodeEngine",
  "jobstatus": ""
},
  {
    "jobId": 1,
    "ackCode" : 10201,
    "path": "",
    "type" : "xxx",
    "executor" : "NodeEngine",
    "jobstatus": ""
  }];*/
var fs = require("fs");
const path = require("path");
const proxy = require('../ws');
var Log = require("../developer/log");
var Script = require("./Script");
const TAG = "[loader_ScriptManager] ";
const RUN_TAG = "[FC_Running_Result]";
var _instance = null;//定义初始化_instance
var exeBegin = true;

var ScriptManager = function () {
    _instance = this;
    this.scripts = {};
};
ScriptManager.ACTION_ADD_JOB = 10201; // 增加场景
ScriptManager.ACTION_DELETE_JOB = 10204; // 删除场景
ScriptManager.EXEUTE_RESULT = {
    JOB_DELETE_SUCCESS: 0,
    JOB_NOT_RUN: 1,
    JOB_SUCCEED_RUNNING: 2,
    JOB_EXECUTE_END: 3,
    JOB_EXECUTE_FAILED: 4,
    JOB_EXECUTE_HANG: 5
};

ScriptManager.getInstance = function () {
    if (_instance != null) {
        return _instance;
    } else {
        return new ScriptManager();
    }
};

/**
 * 要执行的job 为configuration.json 单个元素
 * @param  {[Object]} 单条config，格式如下：
 * {
    "jobId": 0,
    "ackCode" : 10201,
    "path": "",
    "type" : "xxx",
    "executor" : "yyy",
    "jobstatus": ""  //比原来job多的一个字段,
    "groupId" : "xxx",
    "parentGroupIds" : ["901", "hotel"],
    "sceneId" : "xxx"
    }
 */
ScriptManager.prototype.execute = function (job, jobConfig, callback) {
    if( !exeBegin) {
        Log.LOGI(TAG, "exeBegin = " + exeBegin + " will wait");
        return;
    }
    var ackCode = job.ackCode;
    Log.LOGI(TAG, "execute begin...jobid = " + job.jobId + " job.ackCode = " + job.ackCode + "\n");
    var self = this;
    if (ackCode === ScriptManager.ACTION_DELETE_JOB) {
        this.deleteJob(job, true);
    }
    if (ackCode === ScriptManager.ACTION_ADD_JOB) {
        // Log.LOGI(TAG, "ACTION_ADD_JOB begin");
        var status;
        var message;
        if(!this.scripts.hasOwnProperty(job.jobId)) {
            this.scripts[job.jobId] = new Script(job, jobConfig);
        }
        var script = this.scripts[job.jobId];
        script.refreshParams(job.event, job.context);
        script.runHandler(function (err, userRet) {
            Log.LOGI(TAG, "runHandler response err =" + err + " userRet = " + userRet);
            if (err) {
                status = ScriptManager.EXEUTE_RESULT.JOB_EXECUTE_FAILED;
                message = err;
                Log.LOGM(RUN_TAG, `[Failed][${job.sceneId}] jobid ${job.jobId} running handler status : Fail result:${message}`);
                // self.sendJobStatus(job.ackCode, job.jobId, status, job.executor, message, 0);
                return callback(err);
            }
            //TODO: userRet 怎么汇报出去
            status = ScriptManager.EXEUTE_RESULT.JOB_SUCCEED_RUNNING;
            message = "execute success";
            // self.sendJobStatus(job.ackCode, job.jobId, status, job.executor, message, 0);
            Log.LOGM(RUN_TAG, `[Succeeded][${job.sceneId}] jobid ${job.jobId} running handler status : Success result:${message}`);
            Log.LOGI(TAG, "userRet = " + userRet);
            return callback(false);
        });
    }
};

 /**
  * require 一个脚本，并且创建一个脚本把它加到管理列表
  * @param job
  * @param jobConfig
  */
 ScriptManager.prototype.requireCode = function (job, jobConfig) {
    Log.LOGI(TAG, "requireCode begin jobId = " + job.jobId);
    if(!this.scripts.hasOwnProperty(job.jobId)) {
        this.scripts[job.jobId] = new Script(job, jobConfig);
    }
    var script = this.scripts[job.jobId];
    var result = script.requireCode();
    return result;
};

/**
 * 发送script的执行状态给jobmanager
 *
 * @param  {[type]} jobstatus [job的执行状态，格式如下：
 * {
        "ackCode" : int
        "jobId" :   long
        "jobStatus" : int // 1：未运行 2:运行中 3:执行完成 4:执行失败 5:被挂起
        "executor" : string
        "result" : string
        "category" : 0/1
    }
 ]
 * @return {[type]}           [description]
 */
ScriptManager.prototype.sendJobStatus = function (ackCode, jobId, jobStatus, executor, result, category, sceneId) {
    Log.LOGI(TAG, "sendJobStatus begin...jobStatus = " + jobStatus);
    Log.LOGI(TAG, "sendJobStatus begin...result = " + result);
    if(typeof result !== 'string') {
        result = result.toString();
    }
    var object = {};
    object["ackCode"] = ackCode;
    object["jobId"] = jobId;
    object["jobStatus"] = jobStatus;
    object["executor"] = executor;
    object["result"] = result;
    object["category"] = category;
    object["sceneId"] = sceneId;
    Log.LOGI(TAG, "sendJobStatus = " + JSON.stringify(object));
    proxy.sendJobExecuteStatus(jobId, object);
};

ScriptManager.prototype.readConfiguration = function (path) {
    console.log(TAG + "readConfiguration begin...");
    var profileStr = fs.readFileSync(path, "utf-8");
    console.log(TAG + " config = " + profileStr);
    try {
        var confObject = JSON.parse(profileStr);
    } catch (e) {
        confObject = [];
        Log.LOGE(TAG, "configuration Exception = " + e.message);
    }
    return confObject;
};

ScriptManager.prototype.writeConfiguration = function (path, str) {
    console.log(TAG + "writeConfiguration begin...");
    // console.log(TAG + "writeConfiguration config =" + str);
    fs.writeFileSync(path, str);
};

ScriptManager.prototype.deleteJob = function (job, notifyJobManager) {
    //暂时不做事情
};

module.exports = ScriptManager.getInstance();